var Immutable, React, Viewer, classnames, diff, div, keycode, locale, pre, recorder, ref, tabs;

diff = require('immutablediff');

React = require('react/addons');

keycode = require('keycode');

Immutable = require('immutable');

classnames = require('classnames');

recorder = require('./recorder');

locale = require('./app/locale');

Viewer = React.createFactory(require('./app/viewer'));

ref = React.DOM, div = ref.div, pre = ref.pre;

tabs = ['action', 'store', 'prev', 'diff'];

module.exports = React.createClass({
  displayName: "actions-recorder-devtools",
  mixins: [React.addons.PureRenderMixin],
  propTypes: {
    store: React.PropTypes.instanceOf(Immutable.Collection).isRequired,
    records: React.PropTypes.instanceOf(Immutable.List).isRequired,
    inital: React.PropTypes.instanceOf(Immutable.Map).isRequired,
    updater: React.PropTypes.func.isRequired,
    pointer: React.PropTypes.number.isRequired,
    isTravelling: React.PropTypes.bool.isRequired,
    language: React.PropTypes.string,
    width: React.PropTypes.number,
    height: React.PropTypes.number
  },
  getDefaultProps: function() {
    return {
      language: 'en',
      width: window.innerWidth,
      height: window.innerHeight
    };
  },
  getInitialState: function() {
    return {
      tab: 'action'
    };
  },
  onTabSelect: function(name) {
    return this.setState({
      tab: name
    });
  },
  onCommit: function() {
    return recorder.dispatch("actions-recorder/commit");
  },
  onReset: function() {
    return recorder.dispatch("actions-recorder/reset");
  },
  onPeek: function(position) {
    return recorder.dispatch("actions-recorder/peek", position);
  },
  onRun: function() {
    return recorder.dispatch("actions-recorder/run");
  },
  onMergeBefore: function() {
    return recorder.dispatch("actions-recorder/merge-before");
  },
  onClearAfter: function() {
    return recorder.dispatch("actions-recorder/clear-after");
  },
  renderItem: function(record, index) {
    var onClick;
    onClick = (function(_this) {
      return function() {
        return _this.onPeek(index);
      };
    })(this);
    return div({
      key: index,
      style: this.styleItem(this.props.pointer === index),
      onClick: onClick
    }, record.get(0));
  },
  renderAction: function() {
    var record;
    record = this.props.records.get(this.props.pointer);
    return Viewer({
      height: this.props.height - 40,
      data: record.get(1)
    });
  },
  renderPrev: function() {
    var result, updater;
    updater = (function(_this) {
      return function(acc, record) {
        return _this.props.updater(acc, record.get(0), record.get(1));
      };
    })(this);
    result = this.props.records.slice(0, this.props.pointer).reduce(updater, this.props.initial);
    return Viewer({
      height: this.props.height - 40,
      data: result
    });
  },
  renderStore: function() {
    var result, updater;
    updater = (function(_this) {
      return function(acc, record) {
        return _this.props.updater(acc, record.get(0), record.get(1));
      };
    })(this);
    result = this.props.records.slice(0, this.props.pointer + 1).reduce(updater, this.props.initial);
    return Viewer({
      height: this.props.height - 40,
      data: result
    });
  },
  renderDiff: function() {
    var changes, prevResult, result, updater;
    updater = (function(_this) {
      return function(acc, record) {
        return _this.props.updater(acc, record.get(0), record.get(1));
      };
    })(this);
    prevResult = this.props.records.slice(0, this.props.pointer).reduce(updater, this.props.initial);
    result = this.props.records.slice(0, this.props.pointer + 1).reduce(updater, this.props.initial);
    changes = diff(prevResult, result);
    return Viewer({
      height: this.props.height - 40,
      data: changes
    });
  },
  renderCurrent: function() {
    return Viewer({
      height: this.props.height - 40,
      data: this.props.store
    });
  },
  renderDetails: function() {
    return div({
      style: this.styleDetails()
    }, div({
      style: this.styleMode()
    }, tabs.map((function(_this) {
      return function(name) {
        var onClick;
        onClick = function() {
          return _this.onTabSelect(name);
        };
        return div({
          style: _this.styleButton(name === _this.state.tab),
          onClick: onClick,
          key: name
        }, name);
      };
    })(this))), (function() {
      switch (this.state.tab) {
        case 'action':
          return this.renderAction();
        case 'prev':
          return this.renderPrev();
        case 'store':
          return this.renderStore();
        case 'diff':
          return this.renderDiff();
      }
    }).call(this));
  },
  render: function() {
    var hint;
    hint = (function(_this) {
      return function(text) {
        return locale.get(text, _this.props.language);
      };
    })(this);
    return div({
      style: this.styleRoot()
    }, div({
      style: this.styleHeader()
    }, div({
      style: this.styleButton(false),
      onClick: this.onCommit
    }, hint('commit')), div({
      style: this.styleButton(false),
      onClick: this.onReset
    }, hint('reset')), div({
      style: this.styleButton(false),
      onClick: this.onMergeBefore
    }, hint('mergeBefore')), div({
      style: this.styleButton(false),
      onClick: this.onClearAfter
    }, hint('clearAfter')), this.props.isTravelling ? div({
      style: this.styleButton(false),
      onClick: this.onRun
    }, hint('run')) : void 0), div({
      style: this.styleViewer()
    }, div({
      style: this.styleMonitor()
    }, this.props.records.map(this.renderItem)), this.props.isTravelling && (this.props.records.get(this.props.pointer) != null) ? this.renderDetails() : this.renderCurrent()));
  },
  styleRoot: function() {
    return {
      background: 'hsla(200,60%,40%,0.8)',
      color: 'white',
      fontFamily: 'Menlo, Consolas, monospace',
      lineHeight: '1.8em',
      display: 'flex',
      height: '100%',
      flexDirection: 'column',
      transitionProperty: 'left, top',
      transitionDuration: '300ms',
      zIndex: 9999,
      position: 'absolute',
      width: '100%',
      height: '100%'
    };
  },
  styleButton: function(isSelected) {
    return {
      display: 'inline-block',
      backgroundColor: isSelected ? 'hsla(0,100%,100%,0.5)' : 'hsla(0,100%,100%,0.2)',
      marginRight: '10px',
      padding: '0 10px',
      fontSize: '14px',
      fontFamily: 'Verdana, Helvetica, sans-serif',
      lineHeight: '30px',
      cursor: 'pointer'
    };
  },
  styleHeader: function() {
    return {
      WebkitUserSelect: 'none',
      marginBottom: '10px'
    };
  },
  styleViewer: function() {
    return {
      flex: 1,
      display: 'flex',
      flexDirection: 'row',
      height: this.props.height - 40,
      position: 'relative'
    };
  },
  styleMonitor: function() {
    return {
      width: '250px',
      marginRight: '10px',
      overflowX: 'hidden',
      paddingBottom: '40px',
      height: this.props.height - 40
    };
  },
  styleItem: function(isPointer) {
    return {
      cursor: 'pointer',
      lineHeight: '30px',
      fontSize: '13px',
      padding: '0 10px',
      backgroundColor: isPointer ? 'hsla(0,100%,100%,0.2)' : 'transparent'
    };
  },
  styleDetails: function() {
    return {
      flex: 1,
      overflow: 'auto',
      display: 'flex',
      flexDirection: 'column'
    };
  },
  styleMode: function() {
    return {
      height: '30px',
      flexDirection: 'row'
    };
  }
});
